<?php

class Kohana_Account {
    
    protected static $_instance;
    protected $_config;
    protected $_model;

    /**
     * Вовзращает экземпляр класса (singleton-паттерн)
     *
     * @return  instance
     */     
    public static function instance() {
        if(!isset(self::$_instance))
        {
            $config = Kohana::$config->load('account');
            self::$_instance = new self($config);
        }
        return self::$_instance;
    }
    
    private function __construct($config = array()) {
        $this->_config = $config;
        $this->_model = new Model_Account;
    }
    
    /**
     * Генерирует уникальный код активации для данных входных параметров.
     *
     * @param  string $FirstName Имя
     * @param  string $SecondName Отчество
     * @param  string $LastName Фамилия
     * @return  string
     */       
    protected function generateActivationCode()
    {
        $activationCode = Text::random('ABDCEFGHJKLMNPQRSTUVWXYZ123456789', 10);
        return UTF8::strtoupper($activationCode);
    }
    
    public function createTeacher($lastName, $firstName, $secondName, $degreeID, $departamentID)
    {
        $activationCode = $this->generateActivationCode();
        $response = $this->_model->createTeacher($lastName, $firstName, $secondName, $degreeID, $departamentID, $activationCode);
        if($response == -1)
            return -1;
        else
            return $activationCode;
    }
    
    public function createTeacherByDepName($lastName, $firstName, $secondName, $departamentName)
    {
        $activationCode = $this->generateActivationCode();
        $response = $this->_model->createTeacherByDepName($lastName, $firstName, $secondName, $departamentName, $activationCode);
        if($response == -1)
            return -1;
        else
            return $activationCode;
    }
    
    public function createStudent($lastName, $firstName, $secondName, $grade, $groupNum, $facultyID)
    {
        $activationCode = $this->generateActivationCode();
        $response = $this->_model->createStudent($lastName, $firstName, $secondName, $grade, $groupNum, $facultyID, $activationCode);
        if($response == -1)
            return -1;
        else
            return $activationCode;
    }
	
    public function createStudentEx($lastName, $firstName, $secondName, $studentGradeNum, $studentGroupNum, $studentDegree, $studentSpec, $facultyID)
    {
        $activationCode = $this->generateActivationCode();
        $response = $this->_model->createStudentEx($lastName, $firstName, $secondName, $studentGradeNum, $studentGroupNum, $studentDegree, $studentSpec, $facultyID, $activationCode);
        if($response == -1)
            return -1;
        else
            return $activationCode;
    }

    public function createSubject($name, $abbr, $facultyID)
    {
        $response = $this->_model->createSubject($name, $abbr, $facultyID);
        if($response == -1)
            return -1;
        else
            return $activationCode;
    }
    
    private function checkTokenLifetime($creationDate)
    {
        $config = Kohana::$config->load('security.securityPolicy');
        return (time() - $creationDate) > $config['recoveryToken']['lifetime'];
    }


    private function checkRecoveryStatus($email)
    {
        $recovery = $this->_model->GetRecoveryInfoByEMail($email);
        $recoveryHandled = array();
        $status = true;
        foreach($recovery as $row)
        {
            if($this->checkTokenLifetime(strtotime($row['Date'])))
            {
                $this->_model->useRecoveryToken($row['Token']);
            }
            else
                $status = false;
        }
        return $status;
    }
    
    public function checkToken($token)
    {
        $config = Kohana::$config->load('security.securityPolicy');
        $recovery = $this->_model->getRecoveryInfoByToken($token)->offsetGet(0);
        if($recovery['isUsed'])
            return false;
        if($this->checkTokenLifetime(strtotime($recovery['Date'])))
        {
            $this->_model->useRecoveryToken($recovery['Token']);
            return false;
        }
        return true;
    }
    
    public function createRecoveryRequest($email)
    {
        $requestToken = sha1($email.time().Cookie::$salt);
        if($this->checkRecoveryStatus($email))
        {
            $this->_model->createRecoveryToken($email, $requestToken);
            $subject =  ASSEMBLY_SYSTEM_NAME.": Восстановление пароля"; 
            $twig = Twig::factory('email/recovery');
            $twig->curl = URL::base(TRUE, 'https');
            $twig->Token = $requestToken;
            $twig->EMail = $email;
            $twig->Subject = $subject;
            $message = $twig->render();
            $headers = 'MIME-Version: 1.0' . "\r\n";
            $headers .= 'Content-type: text/html; charset=utf-8' . "\r\n";
            $headers .=  "From: RatingSystem@no-reply.mmcs.sfedu.ru\r\n";
            $headers .= "Reply-To: ".$email;
            mail($email, $subject, $message, $headers);
        }
    }
    
    public function changePasswordByToken($token, $password)
    {
        $recovery = $this->_model->getRecoveryInfoByToken($token)->offsetGet(0);
        $this->changePassword($recovery['AccountID'], $password);
        $this->_model->useRecoveryToken($token);
    }
    
    public function isLoginExists($login)
    {
        $login_count = $this->_model->getAccNumByLogin($login);
        return $login_count > 0;
    }
    
    public function isMailExists($email)
    {
        $email_count = $this->_model->getAccNumByMail($email);
        return $email_count > 0;
    }
    
    public function changeLogin($id, $newLogin)
    {
        $response = $this->_model->changeLogin($id, $newLogin);
        return $response != -1;
    }
    
    public function changeEMail($id, $newEMail)
    {
        $response = $this->_model->changeMail($id, $newEMail);
        return $response != -1;
    }
    
    public function changePassword($id, $newPassword)
    {
        $response = $this->_model->changePassword($id, $newPassword);
        return $response != -1;
    }
        
    
}